var spellDb = null;
var profileDb = null;
var $$q = null;

angular.module('pre-app', ['ionic']).run(function($q, $ionicPlatform) {
  $$q = $q;
  $ionicPlatform.registerBackButtonAction(function (event) {
    event.preventDefault();
  }, 101);
});
angular.bootstrap(document.body, ['pre-app']);

var initAds = function() {
    if(!admob) {
      console.error("Admob plugin is not intialized!");
      return;
    }

    var adPublisherIds = {
      ios: {
        banner: "ca-app-pub-2942402007869129/3877283693"
      },
      android: {
        banner: "ca-app-pub-2942402007869129/6045884093"
      }
    };

    var admobid = ionic.Platform.isAndroid() ? adPublisherIds.android : adPublisherIds.ios;

    admob.setOptions({
      publisherId: admobid.banner
    });
};

angular.module('app', ['ionic', 'ionic-native-transitions', 'underscore', 'app.controllers', 'app.routes', 'app.services', 'app.directives', 'ngCordova'])
.run(function($ionicPlatform, $state, $ionicPopup) {
  $ionicPlatform.registerBackButtonAction(function (event) {
    event.preventDefault();
    if($state.current.name=="tabsController.profile"){
      var confirmPopup = $ionicPopup.confirm({
        title: 'Exit App',
        template: 'Are you sure you want to exit?'
      });
      confirmPopup.then(function(res) {
        if(res) {
          navigator.app.exitApp();
        }
      });
    }
    else {
      navigator.app.backHistory();
    }
  }, 101);

  AppRate.preferences.useLanguage = 'en';
  var popupInfo = {};
  popupInfo.title = "Rate Pathfinder Spellbook";
  popupInfo.message = "Ok, we hate these stupid “Please rate our app” panels as much as you do, we live and die by your ratings, and of course you can’t rate an app the moment you buy it, since you don’t know what it’s like yet. \n\nSo, in the politest way possible, we’d like to ask—just this once—that if you like our app you go back and rate it, so we can stay in business and continue making cool apps.";
  popupInfo.cancelButtonLabel = "No, thanks";
  popupInfo.laterButtonLabel = "Remind Me Later";
  popupInfo.rateButtonLabel = "Rate Now";
  AppRate.preferences.customLocale = popupInfo;
  AppRate.preferences.promptAgainForEachNewVersion = false;
  if(ionic.Platform.isAndroid()) {
    AppRate.preferences.storeAppURL.android = 'market://details?id=com.tntgames.pathfinderspellbooklite';
  } else {
    AppRate.preferences.storeAppURL.ios = '1120243720';
    AppRate.preferences.openStoreInApp = false;
  }
  AppRate.preferences.usesUntilPrompt = 2;
  AppRate.promptForRating(false);
})
.config(function($ionicNativeTransitionsProvider, $ionicConfigProvider) {
  console.log('Applying Configuration...');
  $ionicNativeTransitionsProvider.enable(ionic.Platform.isAndroid());
  $ionicConfigProvider.scrolling.jsScrolling(ionic.Platform.isAndroid());
  $ionicNativeTransitionsProvider.setDefaultOptions({
    "duration"          : 10
  });
});

var initialized = function() {
  console.log('Database Opened. Begining Initialization...');
  migratingDatabase().then(function() {
    window.setTimeout(function() {
      console.log('Bootstrapping the application...');
      angular.bootstrap(document.body, ['app']);
    }, 150);
  });
}

ionic.Platform.ready(function() {
  if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard) {
    cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
    cordova.plugins.Keyboard.disableScroll(true);
  }
  if (window.StatusBar) {
    StatusBar.styleDefault();
  }

  initAds();
  admob.createBannerView();
  //window.sqlitePlugin.deleteDB({ name: "profile.db", location: 1 });
  profileDb = window.sqlitePlugin.openDatabase({ name: "profile.db", location: 1, androidLockWorkaround: 1 });
  profileDb.executeSql("CREATE TABLE IF NOT EXISTS DatabaseMigrations (id integer primary key autoincrement unique, name text, executed_on timestamp not null default current_timestamp)");
  profileDb.executeSql("CREATE TABLE IF NOT EXISTS Profiles (id integer primary key autoincrement unique, name text, class_id integer not null, is_default integer not null default 0, created timestamp not null default current_timestamp)");
  profileDb.executeSql("CREATE TABLE IF NOT EXISTS ProfileSpells (id integer primary key autoincrement unique, profile_id integer not null, spell_id integer not null, is_memorized integer not null default 0, prepared_count integer not null default 0, used_count integer not null default 0, created timestamp not null default current_timestamp, foreign key(profile_id) references Profiles(id))");
  profileDb.executeSql("CREATE TABLE IF NOT EXISTS ProfileSources (id integer primary key autoincrement unique, profile_id integer not null, source_id integer not null, created timestamp not null default current_timestamp, foreign key(profile_id) references Profiles(id))");

  window.sqlitePlugin.deleteDatabase({ name: "path-finder.db", location: 'default' }, function() {
    window.plugins.sqlDB.copy("path-finder.db", 2, function() {
        spellDb = window.sqlitePlugin.openDatabase({ name: "path-finder.db", location: 'default' }, function() {
          initialized();
        });
    }, function(error) {
        console.error("There was an error copying the database: " + JSON.stringify(error));
    });
  }, function(err) {
    console.error("There was an error deleting the database: " + JSON.stringify(err));
      window.plugins.sqlDB.copy("path-finder.db", 2, function() {
          spellDb = window.sqlitePlugin.openDatabase({ name: "path-finder.db", location: 'default' }, function() {
            initialized();
          });
      }, function(error) {
          console.error("There was an error copying the database: " + JSON.stringify(error));
      });
  });
});

var runMigration = function(migration) {
  var promise = $$q.defer();

  console.log("Running Migration: " + migration.name);
  console.log("------------- START: " + migration.name + " -----------------")
  profileDb.transaction(function(tx) {
    _.each(migration.queries, function(query) {
      if(query != '') {
        console.log('>>>> ' + query);
        tx.executeSql(query);
      }
    });
    tx.executeSql('insert into DatabaseMigrations (name) values (?)', [migration.name]);
  }, function(error) {
    console.log("!!!----------- END: ERROR Begin --------------!!!");
    console.log('Transaction Error: ' + JSON.stringify(error.message));
    console.log("!!!----------- END: ERROR End --------------!!!");
    promise.reject();
  }, function() {
    console.log("-------------- END: OK ------------------");
    promise.resolve();
  });
  return promise.promise;
};

var runMigrations = function(migrations, promise) {
  if(promise == null) {
    promise = $$q.defer();
  }

  if(migrations.length == 0) {
    promise.resolve();
    return promise.promise;
  }

  runMigration(migrations[0]).then(function() {
    migrations.shift();
    runMigrations(migrations, promise);
  });

  return promise.promise;
};

var migratingDatabase = function() {
  var promise = $$q.defer();
  console.log("Migrating Database...");

  profileDb.executeSql("select name from DatabaseMigrations order by executed_on asc", [], function(results) {
    var migrationsRan = [];
    for(var i = 0; i < results.rows.length; i++) {
      migrationsRan.push(results.rows.item(i).name);
    }

    window.resolveLocalFileSystemURL(window.cordova.file.applicationDirectory + "www/migrations/", function(entry) {
      var dirReader = entry.createReader();
      dirReader.readEntries(function(entries) {
        _.sortBy(entries, function(entry) { return entry.name }).reverse();
        var fileReadPromises = [];
        var order = 0;
        _.each(entries, function(entry) {
          if(!_.contains(migrationsRan, entry.name)) {
            var migration = {
              order: order++,
              name: entry.name,
              queries: []
            };

            var readFilePromise = $$q.defer();
            entry.file(function(file) {
                var reader = new FileReader();
                reader.onloadend = function(e) {
                    var queries = this.result.trim().split(";");
                    migration.queries = queries;
                    readFilePromise.resolve(migration);
                }
                reader.readAsText(file);
            });
            fileReadPromises.push(readFilePromise.promise);
          } else {
            console.log("Already ran: " + entry.name + ". Skipping...");
          }
        });

        $$q.all(fileReadPromises).then(function(migrations) {
          _.sortBy(migrations, function(m) { return m.order });
          runMigrations(migrations, null).then(function() {
            promise.resolve();
          });
        });
      }, function(err) {
        console.log('Error reading migrations directory: ' + JSON.stringify(err));
      });
    }, function(err) {
      console.log("Error readying migrations:" + JSON.stringify(err));
    });
  });
  return promise.promise;
};
