angular.module('app.controllers', [])

.controller('profileCtrl', function($scope, $state, $cordovaSQLite, $ionicSideMenuDelegate, $rootScope, $ionicModal, $ionicHistory, $q, ProfilesService) {
  $scope.profile = {};
  $scope.classes = [];
  $scope.mysteries = [];
  $scope.mysterySpells = [];
  $scope.sources = [];
  $scope.knownSpells = [];
  $scope.editingSpells = { value: false };
  $scope.editingProfile = { value: false };

  var init = function() {
    ProfilesService.getCurrentProfile().then(function(currentProfile) {
      $scope.profile = currentProfile;
      if(currentProfile == null) {
        $ionicSideMenuDelegate.toggleLeft(true);
        return;
      }
      loadProfile();
    });
  }

  init();

  var loadProfile = function() {
    $scope.classes = [];
    $scope.mysteries = [];
    $scope.knownSpells = [];
    $scope.sources = [];
    $scope.selectedClass = null;

    var getClasses = "select id, name, pro_only from Classes order by name";
    $cordovaSQLite.execute(spellDb, getClasses, []).then(function(res) {
      for(var i = 0; i < res.rows.length; i++) {
        var clazz = res.rows.item(i);
        $scope.classes.push(clazz);
      }

      var getProfileQuery = "select * from Profiles where id = ?";
      $cordovaSQLite.execute(profileDb, getProfileQuery, [$scope.profile.id]).then(function(res) {
        $scope.profile = res.rows.item(0);

        if($scope.profile.name == "") {
          $scope.editingProfile.value = true;
        }

        $scope.$watch('profile.name', function(nVal, oVal) {
          if($scope.profile == null || nVal == null || nVal == null) {
            return;
          }

          for(var i = 0; i < $scope.classes.length; i++) {
            if($scope.classes[i].id == $scope.profile.class_id) {
              $scope.selectedClass = $scope.classes[i];
              break;
            }
          }

          var updateProfileQuery = "update Profiles set name = ?, is_default = ? where id = ?";
          $cordovaSQLite.execute(profileDb, updateProfileQuery, [$scope.profile.name, $scope.profile.is_default, $scope.profile.id]).then(function(res) {
            //  ProfilesService.setCurrentProfile($scope.profile);
              $rootScope.$emit('onProfileUpdated', $scope.profile);
              $ionicHistory.clearCache();
          }, function (err) {
              console.error(JSON.stringify(err));
          });
        });
      }, function (err) {
          console.error(JSON.stringify(err));
      });
    }, function (err) {
        console.error(JSON.stringify(err));
    });

    var getMysteries = "select id, name, pro_only from Mysteries order by name";
    $cordovaSQLite.execute(spellDb, getMysteries, []).then(function(res) {
      for(var i = 0; i < res.rows.length; i++) {
        var mystery = res.rows.item(i);
        $scope.mysteries.push(mystery);

        if($scope.mysteries[i].id == $scope.profile.mystery_id) {
          $scope.selectedMystery = mystery;
        }
      }
    }, function (err) {
        console.error(JSON.stringify(err));
    });

    var getSources = "select id, name, pro_only from Sources order by sort_order, name";
    $cordovaSQLite.execute(spellDb, getSources, []).then(function(sources) {
      var getProfileSources = "select * from ProfileSources where profile_id = ?";
      $cordovaSQLite.execute(profileDb, getProfileSources, [$scope.profile.id]).then(function(profileSources) {
        for(var i = 0; i < sources.rows.length; i++) {
          var source = sources.rows.item(i);
          source.is_used = false;

          for(var j = 0; j < profileSources.rows.length; j++) {
            if(profileSources.rows.item(j).source_id == source.id) {
              source.is_used = true;
              break;
            }
          }
          $scope.sources.push(source);
        }
      }, function (err) {
        console.error(JSON.stringify(err));
      });
    }, function (err) {
      console.error(JSON.stringify(err));
    });

    getProfileSpells();
  }

  var getProfileSpells = function() {
    var getProfileSpells = "select * from ProfileSpells where profile_id = ?";
    $scope.knownSpells = [];

    if($scope.profile !=  null) {
      $cordovaSQLite.execute(profileDb, getProfileSpells, [$scope.profile.id]).then(function(res) {
        var profileSpells = [];
        var spellIds = [];
        var getSpellsByIds = "select s.id, s.name, ifnull(cs.level, s.sla_level) as level from Spells s left outer join ClassSpells cs on s.id = cs.spell_id and cs.class_id = ? where s.id in (";
        spellIds.push($scope.profile.class_id);
        for(var i = 0; i < res.rows.length; i++) {
          profileSpells.push(res.rows.item(i));
          spellIds.push(res.rows.item(i).spell_id);
          if(i != 0) {
            getSpellsByIds += ",";
          }
          getSpellsByIds += "?";
        }

        getSpellsByIds += ") order by level asc, s.name";

        var lastLevel = null;
        $cordovaSQLite.execute(spellDb, getSpellsByIds, spellIds).then(function(res) {
          for(var i = 0; i < res.rows.length; i++) {
            var spell = res.rows.item(i);
            for(var j = 0; j < profileSpells.length; j++) {
              if(spell.id == profileSpells[j].spell_id) {
                spell.is_memorized = profileSpells[j].is_memorized == 1;
                break;
              }
            }

            if(lastLevel == null || lastLevel != spell.level) {
              lastLevel = spell.level;
              $scope.knownSpells.push({name: spell.level, isDivider: true})
            }
            spell.isDivider = false;
            $scope.knownSpells.push(spell);
          }
          
          getMysterySpells();

        }, function (err) {
            console.error(JSON.stringify(err));
        });
      }, function (err) {
          console.error(JSON.stringify(err));
      });
    }
  }

  var getMysterySpells = function() {
    if($scope.profile.mystery_id != null) {
      var getSpells = "select s.id, s.name from Spells s inner join MysterySpells ms on s.id = ms.spell_id where ms.mystery_id = ? ORDER BY sla_level;";
      $scope.mysterySpells = [];
      
      $cordovaSQLite.execute(spellDb, getSpells, [$scope.profile.mystery_id]).then(function(res) {
        for(var i = 0; i < res.rows.length; i++) {
          var spell = res.rows.item(i);
          spell.learned = false;
          for(var j = 0; j < $scope.knownSpells.length; j++) {
            if(spell.id == $scope.knownSpells[j].id) {
              spell.learned = true;
              break;
            }
          }

          $scope.mysterySpells.push(res.rows.item(i));
        }
      }, function (err) {
          console.error(JSON.stringify(err));
      });
    }
  }

  $scope.setProfileClass = function(selectedClass) {
    if(selectedClass == null) {
      return;
    } else if (selectedClass.pro_only) {
      if(ionic.Platform.isAndroid()) {
        window.open('market://details?id=com.tntgames.pathfinderspellbook', '_system');
      } else {
        window.open('itms-apps://itunes.apple.com/us/app/spellbook-for-pathfinder/id1118206888?ls=1&mt=8');
      }
      return;
    }

    var updateProfileQuery = "update Profiles set class_id = ? where id = ?";
    $cordovaSQLite.execute(profileDb, updateProfileQuery, [selectedClass.id, $scope.profile.id]).then(function(res) {
      $scope.profile.class_id = selectedClass.id;
      ProfilesService.setCurrentProfile($scope.profile);
      $scope.selectedClass = selectedClass;
      $ionicHistory.clearCache();
    }, function (err) {
        console.error(JSON.stringify(err));
    });

  };

  $scope.setProfileMystery = function(selectedMystery) {
    if(selectedMystery != null) {
      if (selectedMystery.pro_only) {
        if(ionic.Platform.isAndroid()) {
          window.open('market://details?id=com.tntgames.pathfinderspellbook', '_system');
        } else {
          window.open('itms-apps://itunes.apple.com/us/app/spellbook-for-pathfinder/id1118206888?ls=1&mt=8');
        }
        return;
      }

      if($scope.profile.mystery_id != null && selectedMystery.id != $scope.profile.mystery_id) {
        for(var i = 0; i < $scope.knownSpells.length; i++) {
          for(var x = 0; x < $scope.mysterySpells.length; x++) {
            if($scope.knownSpells[i].id == $scope.mysterySpells[x].id) {
              $scope.forgetSpell($scope.knownSpells[i], i);
              break;
            }
          }
        }
        
        getProfileSpells();
      }

      var updateProfileQuery = "update Profiles set mystery_id = ? where id = ?";
      $cordovaSQLite.execute(profileDb, updateProfileQuery, [selectedMystery.id, $scope.profile.id]).then(function(res) {
        $scope.profile.mystery_id = selectedMystery.id;
        ProfilesService.setCurrentProfile($scope.profile);
        $scope.selectedMystery = selectedMystery;
        $ionicHistory.clearCache();
        getMysterySpells();
      }, function (err) {
          console.error(JSON.stringify(err));
      });
    }
  };

  $scope.memorizeSpell = function(spell) {
    spell.is_memorized = !spell.is_memorized;
    var updateProfileSpells = "update ProfileSpells set is_memorized = ? where spell_id = ? and profile_id = ?";
    $cordovaSQLite.execute(profileDb, updateProfileSpells, [spell.is_memorized ? 1 : 0, spell.id, $scope.profile.id]);
  };

  $scope.learnSpell = function(spell) {
    
    if(spell.learned  || $scope.profile == null) {
      return;
    }
    var addSpellToProfileQuery = "insert into ProfileSpells (profile_id, spell_id) values (?, ?)";
    
    $cordovaSQLite.execute(profileDb, addSpellToProfileQuery, [$scope.profile.id, spell.id]).then(function(res) {
      spell.learned = true;
      getProfileSpells();

    }, function (err) {
      console.error(JSON.stringify(err));
    });
  };

  $scope.forgetSpell = function(spell, index) {
    var removeProfileSpells = "delete from ProfileSpells where spell_id = ? and profile_id = ?";
    $cordovaSQLite.execute(profileDb, removeProfileSpells, [spell.id, $scope.profile.id]).then(function(res) {
      $scope.knownSpells.splice(index, 1);

      for(var i = 0; i < $scope.mysterySpells.length; i++) {
        if(spell.name == $scope.mysterySpells[i].name) {
          $scope.mysterySpells[i].learned = false;
        }
      }
    }, function(err) {
      console.error(JSON.stringify(err));
    });
  };

  $scope.selectedSpell = function(spellId) {
    $state.go('tabsController.spell', {id: spellId});
  };

  $scope.selectAllSources = function() {
    var removeProfileSources = "delete from ProfileSources where profile_id = ?";
    $cordovaSQLite.execute(profileDb, removeProfileSources, [$scope.profile.id]).then(function(res) {
      var insertPromises = [];
      var addProfileSources = "insert into ProfileSources (profile_id, source_id) values (?, ?)";
      for(var i = 0; i < $scope.sources.length; i++) {
        var source = $scope.sources[i];
        if(source.pro_only) {
          continue;
        }
        insertPromises.push($cordovaSQLite.execute(profileDb, addProfileSources, [$scope.profile.id, source.id]));
      }

      $q.all(insertPromises).then(function() {
        for(var i = 0; i < $scope.sources.length; i++) {
          $scope.sources[i].is_used = $scope.sources[i].pro_only == 0;
        }
      }, function(err) {
        console.error(JSON.stringify(err));
      });
    }, function(err) {
      console.error(JSON.stringify(err));
    });
  };

  $scope.selectNoSources = function() {
    var removeProfileSources = "delete from ProfileSources where profile_id = ?";
    $cordovaSQLite.execute(profileDb, removeProfileSources, [$scope.profile.id]).then(function(res) {
      for(var i = 0; i < $scope.sources.length; i++) {
        $scope.sources[i].is_used = false;
      }

      $rootScope.$emit('onSourcesChanged', null);
      $ionicHistory.clearCache();
    }, function(err) {
      console.error(JSON.stringify(err));
    });
  };

  $scope.selectSource = function(source) {
    if(source.pro_only) {
      if(ionic.Platform.isAndroid()) {
        window.open('market://details?id=com.tntgames.pathfinderspellbook', '_system');
      } else {
        window.open('itms-apps://itunes.apple.com/us/app/spellbook-for-pathfinder/id1118206888?ls=1&mt=8');
      }
      return;
    }
    var addProfileSources = "insert into ProfileSources (profile_id, source_id) values (?, ?)";
    var removeProfileSources = "delete from ProfileSources where profile_id = ? and source_id = ?";

    var query = !source.is_used ? addProfileSources : removeProfileSources;

    $cordovaSQLite.execute(profileDb, query, [$scope.profile.id, source.id]).then(function(res) {
      source.is_used = !source.is_used;
      $rootScope.$emit('onSourcesChanged', source);
      $ionicHistory.clearCache();
    }, function(err) {
      console.error(JSON.stringify(err));
    });
  };

  $ionicModal.fromTemplateUrl('templates/select-sources-modal.html', {
    scope: $scope
  }).then(function(modal) {
    $scope.sourcesModal = modal;
  });

  $ionicModal.fromTemplateUrl('templates/select-class-modal.html', {
    scope: $scope
  }).then(function(modal) {
    $scope.classModal = modal;
  });

  $ionicModal.fromTemplateUrl('templates/select-mystery-modal.html', {
    scope: $scope
  }).then(function(modal) {
    $scope.mysteryModal = modal;
  });

  $ionicModal.fromTemplateUrl('templates/select-mystery-spells-modal.html', {
    scope: $scope
  }).then(function(modal) {
    $scope.mysterySpellsModal = modal;
  });

  $rootScope.$on('onProfileChange', function(event,profile) {
    if(profile != null) {
      $scope.profile = profile;
      loadProfile();
    }
  });

  $rootScope.$on('onProfileDelete', function(event, profile) {
    if(profile != null) {
      if($scope.profile.id == profile.id) {
        $scope.profile = {};
        $scope.classes = [];
        $scope.mysteries = [];
        $scope.mysterySpells = [];
        $scope.sources = [];
        $scope.knownSpells = [];
        $scope.selectedClass = null;
      }
    }
  });
})

.controller('memorizedCtrl', function($scope, $state, $stateParams, $cordovaSQLite, ProfilesService) {
  var profileId = $stateParams.id;
  $scope.editingSpells = {value: false};
  $scope.memorizedSpells = [];
  $scope.usedSpells = {};

  $scope.viewSpell = function(spell) {
    $state.go('tabsController.spell', {id: spell.id});
  };

  ProfilesService.getCurrentProfile().then(function(profile) {
    $scope.profile = profile;

    var getMemorizedSpells = "select * from ProfileSpells where profile_id = ? and is_memorized = 1";
    $cordovaSQLite.execute(profileDb, getMemorizedSpells, [profileId]).then(function(res) {
      var spellIds = [];
      var memorized = [];
      var getSpellsByIds = "select s.id, s.name, ifnull(cs.level, s.sla_level) as level from Spells s left outer join ClassSpells cs on s.id = cs.spell_id and cs.class_id = ? where s.id in (";
      spellIds.push($scope.profile.class_id);
      for(var i = 0; i < res.rows.length; i++) {
        memorized.push(res.rows.item(i));
        spellIds.push(res.rows.item(i).spell_id);
        if(i != 0) {
          getSpellsByIds += ",";
        }
        getSpellsByIds += "?";
      }

      getSpellsByIds += ") order by level asc, s.name";

      $cordovaSQLite.execute(spellDb, getSpellsByIds, spellIds).then(function(res) {
        var spells = [];
        for(var i = 0; i < res.rows.length; i++) {
          var spell = res.rows.item(i);
          spells.push(spell);
        }

        var lastLevel = null;
        for(var i = 0; i < spells.length; i++) {
          for(var j = 0; j < memorized.length; j++) {
            if(memorized[j].spell_id == spells[i].id) {
              spells[i].prepared_count = memorized[j].prepared_count;
              spells[i].used_count = memorized[j].used_count;
              break;
            }
          }

          if(lastLevel == null || lastLevel != spells[i].level) {
            $scope.memorizedSpells.push({name: spells[i].level, isDivider: true, level: spells[i].level});
            lastLevel = spells[i].level;
          }
          spells[i].isDivider = false;
          $scope.memorizedSpells.push(spells[i]);
          $scope.usedSpells[spells[i].level] = $scope.usedSpells[spells[i].level] == null
                                                ? {level: spells[i].level , count: spells[i].used_count }
                                                : { level: spells[i].level, count: $scope.usedSpells[spells[i].level].count + spells[i].used_count };
        }
      }, function (err) {
          console.error(JSON.stringify(err));
      });
    }, function (err) {
        console.error(JSON.stringify(err));
    });
  });

  $scope.recordSpellUsed = function(spell) {
    var updateProfileSpells = "update ProfileSpells set used_count = ? where spell_id = ? and profile_id = ?";
    $cordovaSQLite.execute(profileDb, updateProfileSpells, [(spell.used_count + 1), spell.id, $scope.profile.id]).then(function(res) {
      spell.used_count++;
      $scope.usedSpells[spell.level] = $scope.usedSpells[spell.level] == null ? {level: spell.level , count: 1 } : { level: spell.level, count: $scope.usedSpells[spell.level].count + 1 };
    }, function(err) {
      console.error(JSON.stringify(err));
    });
  };

  $scope.resetUseCounts = function() {
    var updateProfileSpells = "update ProfileSpells set used_count = 0 where profile_id = ?";
    $cordovaSQLite.execute(profileDb, updateProfileSpells, [profileId]).then(function(res) {
      $scope.usedSpells = {};
      for(var i = 0; i < $scope.memorizedSpells.length; i++) {
        $scope.memorizedSpells[i].used_count = 0;
      }
    }, function(err) {
      console.error(JSON.stringify(err));
    });
  };
})

.controller('spellsCtrl', function($rootScope, $scope, $cordovaSQLite, ProfilesService) {
  $scope.spells = [];
  $scope.currentProfile = null;

  ProfilesService.getCurrentProfile().then(function(profile) {
    $scope.currentProfile = profile;
    var getProfileSources = "select * from ProfileSources where profile_id = ?";
    $cordovaSQLite.execute(profileDb, getProfileSources, [profile.id]).then(function(res) {
      var sources = [];
      for(var i = 0; i < res.rows.length; i++) {
        sources.push(res.rows.item(i).source_id);
      }

      var getProfileSpells = "select * from ProfileSpells where profile_id = ?";
      var profileSpells = [];
      $cordovaSQLite.execute(profileDb, getProfileSpells, [profile.id]).then(function(res) {
        for(var i = 0; i < res.rows.length; i++) {
          profileSpells.push(res.rows.item(i).spell_id);
        }
      });

      var getSpells = "select s.id, s.name from Spells s inner join ClassSpells cs on s.id = cs.spell_id where cs.class_id = ? and s.source_id in (";
      getSpells += sources.join(",");
      getSpells += ") order by name asc;";

      $cordovaSQLite.execute(spellDb, getSpells, [profile.class_id]).then(function(res) {
        var lastCharacter = null;
        for(var i = 0; i < res.rows.length; i++) {
          var spell = res.rows.item(i);
          spell.learned = false;
          for(var j = 0; j < profileSpells.length; j++) {
            if(spell.id == profileSpells[j]) {
              spell.learned = true;
              break;
            }
          }

          if (lastCharacter == null || lastCharacter != spell.name.charAt(0)) {
            lastCharacter = spell.name.charAt(0);
            $scope.spells.push({name: lastCharacter, isDivider: true});
          }
          $scope.spells.push(res.rows.item(i));
        }
      }, function (err) {
          console.error(JSON.stringify(err));
      });
    }, function (err) {
      console.error(JSON.stringify(err));
    });
  });

  var addSpellToProfileQuery = "insert into ProfileSpells (profile_id, spell_id) values (?, ?)";
  $scope.learnSpell = function(spell) {
    if(spell.learned  || $scope.currentProfile == null) {
      return;
    }

    $cordovaSQLite.execute(profileDb, addSpellToProfileQuery, [$scope.currentProfile.id, spell.id]).then(function(res) {
      spell.learned = true;
    }, function (err) {
      console.error(JSON.stringify(err));
    });
  };

  $rootScope.$on('onSpellLearned', function(event, spell) {
    for(var i = 0; i < $scope.spells.length; i++) {
      if(spell.id == $scope.spells[i].id) {
        $scope.spells[i].learned = true;
        break;
      }
    }
  });
})

.controller('spellCtrl', function($rootScope, $scope, $stateParams, $cordovaSQLite, ProfilesService) {
  var spellId = $stateParams.id;
  var getSpell = "select s.*, cs.level, so.name as source from Spells s inner join ClassSpells cs on cs.spell_id = s.id inner join Sources so on so.id = s.source_id where s.id = ?";
  var getProfileSpell = "select * from ProfileSpells where profile_id = ? and spell_id = ?";
  $scope.currentProfile = null;
  $scope.spell = null;

  ProfilesService.getCurrentProfile().then(function(currentProfile) {
    $scope.currentProfile = currentProfile;

    $cordovaSQLite.execute(spellDb, getSpell, [spellId]).then(function(res) {
      $scope.spell = res.rows.item(0);
      $cordovaSQLite.execute(profileDb, getProfileSpell, [$scope.currentProfile.id, $scope.spell.id]).then(function(res) {
        $scope.spell.learned = res.rows.length > 0;
      }, function (err) {
          console.error(JSON.stringify(err));
      });
    }, function (err) {
          console.error(JSON.stringify(err));
    });
  });

  var addSpellToProfileQuery = "insert into ProfileSpells (profile_id, spell_id) values (?, ?)";
  $scope.learnSpell = function() {
    if($scope.spell.learned  || $scope.currentProfile == null) {
      return;
    }

    $cordovaSQLite.execute(profileDb, addSpellToProfileQuery, [$scope.currentProfile.id, $scope.spell.id]).then(function(res) {
      $scope.spell.learned = true;
      $rootScope.$emit('onSpellLearned', $scope.spell);
    }, function (err) {
        console.error(JSON.stringify(err));
    });
  };
})

.controller('levelsCtrl', function($rootScope, $scope, $cordovaSQLite, ProfilesService) {
  $scope.spells = [];
  $scope.currentProfile = null;

  ProfilesService.getCurrentProfile().then(function(currentProfile) {
    $scope.currentProfile = currentProfile;
    var getProfileSources = "select * from ProfileSources where profile_id = ?";
    $cordovaSQLite.execute(profileDb, getProfileSources, [currentProfile.id]).then(function(res) {
      var sources = [];
      for(var i = 0; i < res.rows.length; i++) {
        sources.push(res.rows.item(i).source_id);
      }

      var getProfileSpells = "select * from ProfileSpells where profile_id = ?";
      var profileSpells = [];
      $cordovaSQLite.execute(profileDb, getProfileSpells, [currentProfile.id]).then(function(res) {
        for(var i = 0; i < res.rows.length; i++) {
          profileSpells.push(res.rows.item(i).spell_id);
        }
      });
      showLevelsForClass(currentProfile.class_id, sources, profileSpells);
    }, function (err) {
      console.error(JSON.stringify(err));
    });
  });

  function showLevelsForClass(class_id, sources, profileSpells) {
    $scope.spells = [];
    var lastLevel = null;

    var getSpellsByClass = "select cs.level, s.id, s.name, s.school from Spells s inner join ClassSpells cs on cs.spell_id = s.id where cs.class_id = ? and s.source_id in (";
    getSpellsByClass += sources.join(",");
    getSpellsByClass += ") order by cs.level asc, s.name";
    $cordovaSQLite.execute(spellDb, getSpellsByClass, [class_id]).then(function(res) {
      for(var i = 0; i < res.rows.length; i++) {
        var spell = res.rows.item(i);
        spell.learned = false;
        for(var j = 0; j < profileSpells.length; j++) {
          if(spell.id == profileSpells[j]) {
            spell.learned = true;
            break;
          }
        }

        if (lastLevel == null || lastLevel != spell.level) {
          lastLevel = spell.level;
          $scope.spells.push({name: lastLevel, isDivider: true});
        }
        $scope.spells.push(res.rows.item(i));
      }
    }, function (err) {
        console.error(JSON.stringify(err));
    });
  };

  var addSpellToProfileQuery = "insert into ProfileSpells (profile_id, spell_id) values (?, ?)";
  $scope.learnSpell = function(spell) {
    if(spell.learned  || $scope.currentProfile == null) {
      return;
    }

    $cordovaSQLite.execute(profileDb, addSpellToProfileQuery, [$scope.currentProfile.id, spell.id]).then(function(res) {
      spell.learned = true;
    }, function (err) {
      console.error(JSON.stringify(err));
    });
  };

  $rootScope.$on('onSpellLearned', function(event, spell) {
    for(var i = 0; i < $scope.spells.length; i++) {
      if(spell.id == $scope.spells[i].id) {
        $scope.spells[i].learned = true;
        break;
      }
    }
  });
})

.controller('schoolsCtrl', function($rootScope, $scope, $cordovaSQLite, ProfilesService) {
  $scope.schools = [];
  $scope.spells = [];
  $scope.selectedSchool = null;
  $scope.currentProfile = null;
  var sourceIds = [];
  $scope.profileSpells = [];

  ProfilesService.getCurrentProfile().then(function(profile) {
    $scope.currentProfile = profile;
    var getProfileSources = "select * from ProfileSources where profile_id = ?";
    $cordovaSQLite.execute(profileDb, getProfileSources, [profile.id]).then(function(res) {
      sourceIds = [];
      for(var i = 0; i < res.rows.length; i++) {
        sourceIds.push(res.rows.item(i).source_id);
      }

      var getProfileSpells = "select * from ProfileSpells where profile_id = ?";
      $cordovaSQLite.execute(profileDb, getProfileSpells, [profile.id]).then(function(res) {
        for(var i = 0; i < res.rows.length; i++) {
          $scope.profileSpells.push(res.rows.item(i).spell_id);
        }
      });

      getSchools();
    }, function (err) {
      console.error(JSON.stringify(err));
    });
  });

  function getSchools() {
    var getSchools = "select id, name from Schools";
    $cordovaSQLite.execute(spellDb, getSchools, []).then(function(res) {
      for(var i = 0; i < res.rows.length; i++) {
        var school = res.rows.item(i);
        $scope.schools.push(school);
      }
      $scope.selectedSchool = $scope.schools[0];
      $scope.showSpellsForSchool($scope.selectedSchool);
    }, function (err) {
        console.error(JSON.stringify(err));
    });
  }

  $scope.showSpellsForSchool = function(school) {
    var getSpellsBySchool = "select s.* from Spells s inner join ClassSpells cs on s.id = cs.spell_id where cs.class_id = ? and s.school = ? and s.source_id in (";
    getSpellsBySchool += sourceIds.join(",");
    getSpellsBySchool += ") order by name asc";
    $scope.spells = [];
    var lastCharacter = null;

    $cordovaSQLite.execute(spellDb, getSpellsBySchool, [$scope.currentProfile.class_id, school.name]).then(function(res) {
      for(var i = 0; i < res.rows.length; i++) {
        var spell = res.rows.item(i);
        spell.learned = false;
        
        for(var j = 0; j < $scope.profileSpells.length; j++) {
          if(spell.id == $scope.profileSpells[j]) {
            spell.learned = true;
            break;
          }
        }

        if (lastCharacter == null || lastCharacter != spell.name.charAt(0)) {
          lastCharacter = spell.name.charAt(0);
          $scope.spells.push({name: lastCharacter, isDivider: true});
        }
        $scope.spells.push(res.rows.item(i));
      }
    }, function (err) {
        console.error(JSON.stringify(err));
    });
  };

  var addSpellToProfileQuery = "insert into ProfileSpells (profile_id, spell_id) values (?, ?)";
  $scope.learnSpell = function(spell) {
    if(spell.learned  || $scope.currentProfile == null) {
      return;
    }

    $cordovaSQLite.execute(profileDb, addSpellToProfileQuery, [$scope.currentProfile.id, spell.id]).then(function(res) {
      spell.learned = true;
    }, function (err) {
      console.error(JSON.stringify(err));
    });
  };

  $rootScope.$on('onSpellLearned', function(event, spell) {
    for(var i = 0; i < $scope.spells.length; i++) {
      if(spell.id == $scope.spells[i].id) {
        $scope.spells[i].learned = true;
        break;
      }
    }
  });
})

.controller('allSpellsCtrl', function($rootScope, $scope, $cordovaSQLite, ProfilesService) {
  $scope.spells = [];
  $scope.currentProfile = null;

  ProfilesService.getCurrentProfile().then(function(profile) {
    $scope.currentProfile = profile;
    var getProfileSources = "select * from ProfileSources where profile_id = ?";
    $cordovaSQLite.execute(profileDb, getProfileSources, [profile.id]).then(function(res) {
      var sources = [];
      for(var i = 0; i < res.rows.length; i++) {
        sources.push(res.rows.item(i).source_id);
      }

      var getProfileSpells = "select * from ProfileSpells where profile_id = ?";
      var profileSpells = [];
      $cordovaSQLite.execute(profileDb, getProfileSpells, [profile.id]).then(function(res) {
        for(var i = 0; i < res.rows.length; i++) {
          profileSpells.push(res.rows.item(i).spell_id);
        }
      });

      var getSpells = "select s.id, s.name from Spells s where s.source_id in (";
      getSpells += sources.join(",");
      getSpells += ") order by name asc;";

      $cordovaSQLite.execute(spellDb, getSpells, []).then(function(res) {
          var lastCharacter = null;
          for(var i = 0; i < res.rows.length; i++) {
            var spell = res.rows.item(i);
            spell.learned = false;
            for(var j = 0; j < profileSpells.length; j++) {
              if(spell.id == profileSpells[j]) {
                spell.learned = true;
                break;
              }
            }

            if (lastCharacter == null || lastCharacter != spell.name.charAt(0)) {
              lastCharacter = spell.name.charAt(0);
              $scope.spells.push({name: lastCharacter, isDivider: true});
            }
            $scope.spells.push(res.rows.item(i));
          }
      }, function (err) {
          console.error(JSON.stringify(err));
      });
    }, function (err) {
      console.error(JSON.stringify(err));
    });
  });

  var addSpellToProfileQuery = "insert into ProfileSpells (profile_id, spell_id) values (?, ?)";
  $scope.learnSpell = function(spell) {
    if(spell.learned  || $scope.currentProfile == null) {
      return;
    }

    $cordovaSQLite.execute(profileDb, addSpellToProfileQuery, [$scope.currentProfile.id, spell.id]).then(function(res) {
      spell.learned = true;
    }, function (err) {
      console.error(JSON.stringify(err));
    });
  };

  $rootScope.$on('onSpellLearned', function(event, spell) {
    for(var i = 0; i < $scope.spells.length; i++) {
      if(spell.id == $scope.spells[i].id) {
        $scope.spells[i].learned = true;
        break;
      }
    }
  });
})
